<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
namespace Drupal\commerce_tigo_money\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_checkout\CheckoutOrderManagerInterface;
use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_price\Price;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_tigo_money\ApiClient;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * @CommercePaymentGateway(
 *   id = "tigo_money_redirect_checkout",
 *   label = @Translation("Tigo Money OffSite."),
 *   display_label = @Translation("Tigo Money"),
 *   forms = {
 *    "offsite-payment" = "Drupal\commerce_tigo_money\PluginForm\TigoMoneyOffsite\PaymentForm",
 *    "capture-payment" = "Drupal\commerce_payment\PluginForm\PaymentCaptureForm",
 *    "void-payment" = "Drupal\commerce_payment\PluginForm\PaymentVoidForm",
 * },
 * modes= {
 *     "test" = "Test",
 *     "live" = "Live"
 * },
 * payment_method_types = {"credit_card"},
 * )
 */

class TigoMoneyPaymentGateway extends OffsitePaymentGatewayBase implements TigoMoneyPaymentGatewayInterface {

    use StringTranslationTrait;

    /**
     * The checkout order manager.
     *
     * @var \Drupal\commerce_checkout\CheckoutOrderManagerInterface
     */
    protected $checkoutOrderManager;

    /**
     * The state manager.
     *
     * @var \Drupal\Core\State\StateInterface
     */
    protected $state;

    /**
     * {@inheritdoc}
     */
    public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time, CheckoutOrderManagerInterface $checkout_order_manager, StateInterface $state) {
        parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);

        $this->checkoutOrderManager = $checkout_order_manager;
        $this->state = $state;
    }

    /**
     * {@inheritdoc}
     */
    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
        return new static(
            $configuration,
            $plugin_id,
            $plugin_definition,
            $container->get('entity_type.manager'),
            $container->get('plugin.manager.commerce_payment_type'),
            $container->get('plugin.manager.commerce_payment_method_type'),
            $container->get('datetime.time'),
            $container->get('commerce_checkout.checkout_order_manager'),
            $container->get('state')
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getPublicKey() {
        return $this->configuration['api_key'];
    }

    /**
     * {@inheritdoc}
     */
    public function getSecretKey() {
        return $this->configuration['api_secret'];
    }

    /**
     * {@inheritdoc}
     */
    public function getStoreName() {
        return $this->configuration['account_name'];
    }

    /**
     * {@inheritdoc}
     */
    public function getAccount() {
        return $this->configuration['account'];
    }

    /**
     * {@inheritdoc}
     */
    public function getPin() {
        return $this->configuration['pin'];
    }

    /**
     * @inheritdoc
     */
    public function defaultConfiguration() {
        return [
            'account_name' => 'Amazon',
            'account' => '0981123456',
            'pin' => '1234',
            'api_key' => 'yFqqXJh9PqnugfrsdfffDff3sf556',
            'api_secret' => '0808sEWdffASvvdd355sskl',
        ] + parent::defaultConfiguration();
    }

    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
        $form = parent::buildConfigurationForm($form, $form_state); // TODO: Change the autogenerated stub

        $form['account_name'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Agent Store Name'),
            '#default_value' => $this->configuration['account_name'],
            '#description' => $this->t('Your store name.'),
            '#required' => true,
        ];

        $form['account'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Agent Account'),
            '#default_value' => $this->configuration['account'],
            '#description' => $this->t('Your tigo money phone number acredited.'),
            '#required' => true,
        ];
        $form['pin'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Agent PIN'),
            '#default_value' => $this->configuration['pin'],
            '#description' => $this->t('Your tigo money agent pin.'),
            '#required' => true,
        ];
        $form['api_key'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Api key'),
            '#default_value' => $this->configuration['api_key'],
            '#description' => $this->t('Unique client identifier assigned during the registration process with Tigo Money.'),
            '#required' => true,
        ];
        $form['api_secret'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Secret key'),
            '#default_value' => $this->configuration['api_secret'],
            '#description' => $this->t('Secret password provided during the registration process with Tigo Money.'),
            '#required' => true,
        ];

        return $form;
    }

    /**
     * {@inheritdoc}
     */

    public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
        parent::submitConfigurationForm($form, $form_state); // TODO: Change the autogenerated stub

        if (!$form_state->getErrors()) {
            $values = $form_state->getValue($form['#parents']);
            $this->configuration['account_name'] = $values['account_name'];
            $this->configuration['account'] = $values['account'];
            $this->configuration['pin'] = $values['pin'];
            $this->configuration['api_key'] = $values['api_key'];
            $this->configuration['api_secret'] = $values['api_secret'];
        }
    }

    /**
     * {@inheritdoc}
     */
    public function buildPaymentOperations(PaymentInterface $payment) {
        $operations = [];
        $operations['capture'] = [
            'title' => $this->t('Capture'),
            'page_title' => $this->t('Capture payment'),
            'plugin_form' => 'capture-payment',
            'access' => $this->canCapturePayment($payment),
        ];
        $operations['void'] = [
            'title' => $this->t('Void'),
            'page_title' => $this->t('Void payment'),
            'plugin_form' => 'void-payment',
            'access' => $this->canVoidPayment($payment),
        ];

        return $operations;
    }

    /**
     * {@inheritdoc}
     */
    public function canCapturePayment(PaymentInterface $payment) {
        return $payment->getState()->getId() === 'authorization';
    }

    /**
     * {@inheritdoc}
     */
    public function capturePayment(PaymentInterface $payment, Price $amount = null) {
        $this->assertPaymentState($payment, ['authorization']);

        // If not specified, use the entire amount.
        $amount = $amount ?: $payment->getAmount();
        $payment->state = 'completed';
        $payment->setAmount($amount);
        $payment->save();
    }

    /**
     * {@inheritdoc}
     */
    public function canVoidPayment(PaymentInterface $payment) {
        return $payment->getState()->getId() === 'authorization';
    }

    /**
     * {@inheritdoc}
     */
    public function voidPayment(PaymentInterface $payment) {
        $this->assertPaymentState($payment, ['authorization']);

        $payment->state = 'voided';
        $payment->save();
    }

    /**
     * {@inheritdoc}
     */
    public function onNotify(Request $request) {
        if (!$response_data = $request->getContent()) {
            throw new PaymentGatewayException('Notify: Response data missing, aborting.');
        }

        $response_data = $request->request->all();

        if (empty($response_data['merchantTransactionId'])) {
            throw new PaymentGatewayException('Invoice id missing, aborting.');
        }
        /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
        if (!$order = Order::load($response_data['merchantTransactionId'])) {
            throw new PaymentGatewayException('Could not find matching order. ID= ' . $response_data['merchantTransactionId']);
        }

        $logger = \Drupal::logger('commerce_tigo_money');
        $logger->info('Return: Request content: ' . $request->getContent());

        $public_key = $this->getPublicKey();
        $secret_key = $this->getSecretKey();
        $agent_account = $this->getAccount();
        $agent_pin = $this->getPin();
        $agent_account = $this->getAccount();
        $agent_name = $this->getStoreName();
        $gateway_mode = $this->getMode();

        $tigo_money = new ApiClient($public_key, $secret_key, $agent_account, $agent_pin, $agent_name);

        if ($gateway_mode == 'live') {
            $tigo_money->sandboxMode(false);
        } else {
            $tigo_money->sandboxMode(true);
        }

        $mfst_id = $response_data['mfsTransactionId'];
        $order_id = $order->id();
        $transaction_data = $tigo_money->getTransaction($mfst_id, $order_id);
        $transaction_status = $transaction_data['Transaction']['status'];

        if ($transaction_status === 'success') {
            $ammount = $order->getTotalPrice()->getNumber() + 0;
            $charged_amount = $transaction_data['originPayment']['amount'];

            if ($ammount == $charged_amount) {
                $state_item = $order->get('state')->first();
                $current_state = $state_item->getValue();

                if ($current_state['value'] !== 'draft') {
                    $logger->info('Notify: Skipping order state transition, order already not in "draft" state anymore, current state: @current-state', ['@current-state' => $current_state['value']]);
                    return false;
                }

                if ($transitions = $state_item->getTransitions()) {
                    $state_item->applyTransition(current($transitions));
                    // Unlock the order if needed.
                    $order->isLocked() ? $order->unlock() : null;
                    $order->save();
                    $logger->info('Notify: Set transition successfully.');
                }

                $tigo_ref = $transaction_data['Transaction']['mfsTransactionId'];

                $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
                $payment = $payment_storage->create([
                    'state' => 'authorization',
                    'amount' => $order->getTotalPrice(),
                    'payment_gateway' => $this->parentEntity->id(),
                    'order_id' => $order_id,
                    'remote_id' => $tigo_ref,
                    'remote_state' => $transaction_status,
                ]);

                $payment->save();

                $this->messenger()->addStatus($this->t('Transaction ' . $transaction_status));

                $logger->info('Notify: Payment information saved successfully. Transaction reference: ' . $order_id);
            } else {
                $logger->warning('Notify: Charged Amount is: ' . $charged_amount . ' while Order Amount: ' . $ammount);
                $this->messenger()->addWarning($this->t(' Error there was a problem with your payment..'));
            }
        } else {
            $logger->warning('Notify: Tigo Money status fail, orderID: ' . $order_id);
            $this->messenger()->addWarning($this->t('Transaction ' . $transaction_status));
        }
    }

    /**
     * {@inheritdoc}
     */
    public function onReturn(OrderInterface $order, Request $request) {
        $logger = \Drupal::logger('commerce_tigo_money');
        $logger->info('Return: Request content: ' . $request->getContent());

        if (!$order->isPaid()) {
            throw new PaymentGatewayException('Return: The order is not paid.');
        }
    }
}
